SUBROUTINE PHYMFPOS(KPROMA,KPROMDLR,KST,KND,KFLDAUX,KFLDSPP,KGFL,YDIN,YDGFL,&
 & POROG,PGM,PGEMU,PGELAM,PUT0,PVT0,PTT0,PSPDT0,PGRHL,PWW,PGFL,PSPT0,&
 & PSP_RR,PSP_SG,PSD_VF,PSD_VV, &
 & PXTCLS,PXRHCLS,PXQCLS,PXUCLS,PXVCLS,&
 & PAUX,PGPP)  

!**** *PHYMFPOS* - post-processing

!     PURPOSE.
!     -------
!           Physico-dynamic post-processing interface for Meteo-France.

!**   INTERFACE.
!     ---------
!        *CALL* *PHYMFPOS(...)

!        EXPLICIT ARGUMENTS :
!        ------------------
!        * INPUT :
!        KPROMA    : horizontal dimension 
!        KPROMDLR  : horizontal dimension for LRWSDLR option arrays.
!        KST       : first point to post-process 
!        KND       : last point to post-process 
!        KFLDAUX   : number of fields in output array (not fitted)
!        KFLDSPP   : number of fields in output array (fitted)
!        KGFL      : Number of GFL fields in PGFL
!        YDIN      : Pointers allowing to retrieve individual fields in PGFL
!        YDGFL     : says if GFL is active or not
!        POROG     : Surface geopotential
!        PGM       : Map factor
!        PGEMU     : Sine of geographic latitude
!        PGELAM    : geographic longitude
!        PUT0      : zonal wind time t
!        PVT0      : meridional wind time t
!        PTT0      : temperature time t
!        PSPDT0    : pressure departure variable if LFPART2=F;
!                    pressure departure "pre-prehyd" if LFPART2=T.
!        PGRHL     : Graupels+Hail
!        PWW       : geometric vertical velocity from the model
!        PGFL      : GFL
!        PSPT0     : surface pressure variable time t
!                    (contains log(presims) if lrwsdlr=T)
!        PSP_RR    : RR=RESVR surface buffer (soil quantities for the surface
!                    and sometimes the first (upper) reservoir)
!        PSP_SG    : SG=SNOWG surface buffer (quantities linked to surface snow)
!        PSD_VF    : VF=VARSF surface buffer (climatological/geographical parameters)
!        PSD_VV    : VV=VCLIV surface buffer (vegetation parameters)
!        PXTCLS    : Tcls from model
!        PXRHCLS   : RHcls from model
!        PXQCLS    : QCLS from model
!        PXUCLS    : Ucls from model
!        PXVCLS    : Vcls from model

!        * OUTPUT (post-processed fields) :
!        PAUX      : fields to remain gridpoint
!        PGPP      : fields to be spectrally fitted

!     EXTERNALS.
!     ---------

!     AUTHOR.
!     -------
!        RYAD EL KHATIB *METEO-FRANCE*

!     MODIFICATIONS.
!     -------------
!   ORIGINAL : 2000-10-06
!   R. El Khatib : 01-01-05 plug-in FPCICA instead of FPCAPE
!   R. El Khatib : 01-08-07 Pruning options
!   R. El Khatib : 03-08-12 gfl
!   R. El Khatib : 03-04-17 Fullpos improvemnts
!   M.Hamrud      01-Oct-2003 CY28 Cleaning
!   N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!   G. Hello     : 07-07 MOdel radiances
!   K. Yessad (Aug 2009): rewrite vertical interpolator in post-processing.
!   Y. Bouteloup : 01-05-2009 Add Total (radiative) cloud water and ice
!   F. Bouyssel  : 15-07-2009 Add LFPCAPEX
!   Y. Seity     : 15-06-2010 correction for ISPs in AROME Fullpos offline
!   C. Wittmann  : 28-06-2013 Add call to lightening diagnostics
!   J. Cedilnik  : 19-08-2013 Add call to storm motion diagnostics, helicity, vert. shear

!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK

USE YOMCT0   , ONLY : LFPART2  ,LNHDYN, NSTOP
USE YOMDIM   , ONLY : NFLEVG
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMGEM   , ONLY : YRVAB
USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,RCW      ,RCS      , &
 & RLVTT    ,RLSTT    ,RTT      ,RALPW    ,RBETW    ,RGAMW    , &
 & RALPS    ,RBETS    ,RGAMS    ,RALPD    ,RBETD    ,RGAMD      
USE YOMPHY   , ONLY : LNEIGE
USE YOMFPC   , ONLY : NFPCAPE  ,CFPFMT, LMOCONVAR, LFPCAPEX
USE YOMMTS   , ONLY : LMTS
USE YOMVPOS  , ONLY : QFPTYPE
USE YOMAFN   , ONLY : TFP_UCLS ,TFP_VCLS ,TFP_TCLS ,TFP_QCLS ,TFP_RCLS ,&
 & TFP_FCLS ,TFP_CAPE ,TFP_CIEN ,TFP_UGST ,TFP_VGST ,TFP_FGST ,&
 & TFP_SMC  ,TFP_ASMC ,TFP_DYNDS,TFP_LCL, TFP_FCL , TFP_EL ,&
 & TFP_STRMMU ,TFP_STRMMV ,TFP_DLS ,TFP_LLS ,TFP_SRH , &
 & TFP_FLASH ,TFP_MSAT7C1,TFP_MSAT7C2,&
 & TFP_MSAT8C1,TFP_MSAT8C2,TFP_MSAT8C3,TFP_MSAT8C4,&
 & TFP_MSAT8C5,TFP_MSAT8C6,TFP_MSAT8C7,TFP_MSAT8C8,&
 & TFP_MSAT9C1,TFP_MSAT9C2,TFP_MSAT9C3,TFP_MSAT9C4,&
 & TFP_MSAT9C5,TFP_MSAT9C6,TFP_MSAT9C7,TFP_MSAT9C8,&
 & TFP_GOES11C1,TFP_GOES11C2,TFP_GOES11C3,TFP_GOES11C4,&
 & TFP_GOES12C1,TFP_GOES12C2,TFP_GOES12C3,TFP_GOES12C4,&
 & TFP_MTSAT1C1,TFP_MTSAT1C2,TFP_MTSAT1C3,TFP_MTSAT1C4  
USE SURFACE_FIELDS_MIX  , ONLY :  &
 & YSP_SGD, YSP_SG, YSP_RRD, YSP_RR, &
 & YSD_VFD, YSD_VF, YSD_VVD, YSD_VV
USE TYPE_GFLFLDS, ONLY: TYPE_IGFLFLDD,TYPE_LGFLFLDD
USE PAR_RDLR , ONLY : JP_DIMR0

!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMA 
INTEGER(KIND=JPIM),INTENT(IN)    :: KPROMDLR
INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDAUX
INTEGER(KIND=JPIM),INTENT(IN)    :: KFLDSPP
INTEGER(KIND=JPIM),INTENT(IN)    :: KGFL
TYPE(TYPE_IGFLFLDD),INTENT(IN)   :: YDIN
TYPE(TYPE_LGFLFLDD),INTENT(IN)   :: YDGFL
REAL(KIND=JPRB)   ,INTENT(IN)    :: POROG(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGM(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGEMU(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGELAM(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PUT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PVT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PTT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPDT0(KPROMA,NFLEVG) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGRHL(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PWW(KPROMA,NFLEVG)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(KPROMA,NFLEVG,KGFL)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSPT0(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP_RR(KPROMA,YSP_RRD%NDIM)
REAL(KIND=JPRB)   ,INTENT(INOUT) :: PSP_SG(KPROMA,YSP_SGD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSD_VF(KPROMA,YSD_VFD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PSD_VV(KPROMA,YSD_VVD%NDIM)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXTCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXRHCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXQCLS(KPROMA)
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXUCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PXVCLS(KPROMA) 
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PAUX(KPROMA,KFLDAUX)
REAL(KIND=JPRB)   ,INTENT(OUT)   :: PGPP(KPROMA,KFLDSPP)

!     ------------------------------------------------------------------

! === LOGICAL VARIABLES COMPUTED IN PART 1 =============================
LOGICAL :: LLPBL  ! Computation of PBL fields
LOGICAL :: LLICV  ! to activate the computation of CAPE & CIEN
LOGICAL :: LLSMC  ! to activate the computation of Surface MOCON
LOGICAL :: LLASMC ! to activate the computation of analysed surface MOCON
LOGICAL :: LLMTS  ! to activate the computation of model radiances
LOGICAL :: LLSTRMM ! to activate the computation of storm motion diagnostics 
LOGICAL :: LLSHEAR ! to activate the computation of shear 
LOGICAL :: LLSRH   ! to activate the computation of storm relative helicity 
LOGICAL :: LLFLASH ! to activate the computation of flash diagnostic

! === REAL VARIABLES COMPUTED IN PART 2.1 ==============================
REAL(KIND=JPRB) :: ZUT0(KPROMA,NFLEVG),ZVT0(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZTT0(KPROMA,NFLEVG),ZPDT0(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZQT0(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZLT0(KPROMA,NFLEVG),ZIT0(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZLRADT0(KPROMA,NFLEVG),ZIRADT0(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRRT0(KPROMA,NFLEVG),ZSNT0(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGRHLT0(KPROMA,NFLEVG),ZAT0(KPROMA,NFLEVG)

! === REAL VARIABLES COMPUTED IN PART 2.2 ==============================
REAL(KIND=JPRB) :: ZCP(KPROMA,NFLEVG),ZR(KPROMA,NFLEVG),ZUSL_KAP(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZPRESH(KPROMA,0:NFLEVG),ZPRESF(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZDELP(KPROMA,NFLEVG),ZRDELP(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZLNPR(KPROMA,NFLEVG),ZALPH(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRTGR(KPROMA,NFLEVG),ZRPRE(KPROMA,NFLEVG),ZRPP(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZNHPREF(KPROMA,NFLEVG),ZRNHPPI(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZUSL_NHPPI(KPROMA,NFLEVG),ZUSL_PDEP(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRRED(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZGEOPH(KPROMA,0:NFLEVG),ZGEOPF(KPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZRDLR(KPROMDLR,0:NFLEVG,JP_DIMR0)

! === VARIABLES COMPUTED IN PART 2.3 ===================================
! * Working arrays:
REAL(KIND=JPRB) :: ZXQCLS(KPROMA) ! (analysed) specific moisture at PBL height
REAL(KIND=JPRB) :: ZTS0(KPROMA) ! local soil temperature 
REAL(KIND=JPRB) :: Z1SGM(KPROMA)  ! Reverse map factor
REAL(KIND=JPRB) :: ZQO3(KPROMA,0:NFLEVG)
! * Local post-processed fields:
REAL(KIND=JPRB) :: ZUCLS(KPROMA),ZVCLS(KPROMA),ZFCLS(KPROMA),ZTCLS(KPROMA)
REAL(KIND=JPRB) :: ZUGST(KPROMA),ZVGST(KPROMA),ZFGST(KPROMA),ZUSMC(KPROMA)
REAL(KIND=JPRB) :: ZQCLS(KPROMA),ZRHCLS(KPROMA),ZPCLS(KPROMA),ZVSMC(KPROMA)
REAL(KIND=JPRB) :: ZCAPE(KPROMA),ZCIN(KPROMA),ZUASM(KPROMA),ZVASM(KPROMA)
REAL(KIND=JPRB) :: ZSTRMMU(KPROMA),ZSTRMMV(KPROMA)
REAL(KIND=JPRB) :: ZFLASH(KPROMA)
REAL(KIND=JPRB) :: ZDLS(KPROMA),ZLLS(KPROMA),ZSRH(KPROMA)
REAL(KIND=JPRB) :: ZTBMSAT7(KPROMA,2),ZTBMSAT8(KPROMA,8),ZTBMSAT9(KPROMA,8)
REAL(KIND=JPRB) :: ZTBGOES11(KPROMA,4),ZTBGOES12(KPROMA,4),ZTBMTSAT1(KPROMA,4)
INTEGER(KIND=JPIM) :: ILCL(KPROMA) ! Lifting condensation level
INTEGER(KIND=JPIM) :: IFCL(KPROMA) ! Free convection level
INTEGER(KIND=JPIM) :: IEL(KPROMA)  ! Equilibrium level
REAL(KIND=JPRB) :: ZLCL(KPROMA) !  Lifting condensation level (height)
REAL(KIND=JPRB) :: ZFCL(KPROMA) ! Free convection level (height)
REAL(KIND=JPRB) :: ZEL(KPROMA)  ! Equilibrium level (height)

! === OTHER VARIABLES ==================================================
REAL(KIND=JPRB) :: ZDELTA, ZES
INTEGER(KIND=JPIM) :: JLEV, ICAPE, JROF
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "abor1.intfb.h"
#include "fpachmt.intfb.h"
#include "fpcica.intfb.h"
#include "gpgeo.intfb.h"
#include "gnhdlr.intfb.h"
#include "gnhdlrb.intfb.h"
#include "gnhpre.intfb.h"
#include "gppref.intfb.h"
#include "gppreh.intfb.h"
#include "gprcp.intfb.h"
#include "gpxyb.intfb.h"
#include "mts_phys.intfb.h"
#include "fcttrm.h"
#include "fpsrh.intfb.h"
#include "fpstrmm.intfb.h"
#include "fpshear.intfb.h"
#include "fpdiagflash.intfb.h"

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('PHYMFPOS',0,ZHOOK_HANDLE)
!     ------------------------------------------------------------------

!*       1.    INITIALIZATIONS.
!              ---------------

LLSMC =QFPTYPE%LL(TFP_SMC%ICOD)
LLASMC=QFPTYPE%LL(TFP_ASMC%ICOD)
LLICV = (CFPFMT == 'MODEL') .AND. &
 & (QFPTYPE%LL(TFP_CAPE%ICOD) .OR. QFPTYPE%LL(TFP_CIEN%ICOD))  
LLSTRMM = QFPTYPE%LL(TFP_STRMMU%ICOD) .OR. QFPTYPE%LL(TFP_STRMMV%ICOD)
LLSHEAR = QFPTYPE%LL(TFP_LLS%ICOD) .OR. QFPTYPE%LL(TFP_DLS%ICOD)
LLSRH   = QFPTYPE%LL(TFP_SRH%ICOD)   
LLFLASH = QFPTYPE%LL(TFP_FLASH%ICOD)
LLMTS = QFPTYPE%LL(TFP_MSAT7C1%ICOD) .OR. QFPTYPE%LL(TFP_MSAT7C2%ICOD) .OR. &
      & QFPTYPE%LL(TFP_MSAT8C1%ICOD) .OR. QFPTYPE%LL(TFP_MSAT8C2%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT8C3%ICOD) .OR. QFPTYPE%LL(TFP_MSAT8C4%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT8C5%ICOD) .OR. QFPTYPE%LL(TFP_MSAT8C6%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT8C7%ICOD) .OR. QFPTYPE%LL(TFP_MSAT8C8%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT9C1%ICOD) .OR. QFPTYPE%LL(TFP_MSAT9C2%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT9C3%ICOD) .OR. QFPTYPE%LL(TFP_MSAT9C4%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT9C5%ICOD) .OR. QFPTYPE%LL(TFP_MSAT9C6%ICOD).OR. &
      & QFPTYPE%LL(TFP_MSAT9C7%ICOD) .OR. QFPTYPE%LL(TFP_MSAT9C8%ICOD).OR. &
      & QFPTYPE%LL(TFP_GOES11C1%ICOD) .OR. QFPTYPE%LL(TFP_GOES11C2%ICOD).OR. &
      & QFPTYPE%LL(TFP_GOES11C3%ICOD) .OR. QFPTYPE%LL(TFP_GOES11C4%ICOD).OR. &
      & QFPTYPE%LL(TFP_GOES12C1%ICOD) .OR. QFPTYPE%LL(TFP_GOES12C2%ICOD).OR. &
      & QFPTYPE%LL(TFP_GOES12C3%ICOD) .OR. QFPTYPE%LL(TFP_GOES12C4%ICOD).OR. &
      & QFPTYPE%LL(TFP_MTSAT1C1%ICOD) .OR. QFPTYPE%LL(TFP_MTSAT1C2%ICOD).OR. &
      & QFPTYPE%LL(TFP_MTSAT1C3%ICOD) .OR. QFPTYPE%LL(TFP_MTSAT1C4%ICOD)
LLMTS = LLMTS .AND. LMTS
LLPBL = LLSMC .OR. LLASMC .OR. (LLICV .AND. (.NOT.LFPCAPEX)) .OR.  &
 & ((CFPFMT == 'MODEL') .AND.                                     &
 & QFPTYPE%LL(TFP_UCLS%ICOD) .OR. QFPTYPE%LL(TFP_VCLS%ICOD) .OR. &
 & QFPTYPE%LL(TFP_TCLS%ICOD) .OR. QFPTYPE%LL(TFP_QCLS%ICOD) .OR. &
 & QFPTYPE%LL(TFP_RCLS%ICOD) .OR. QFPTYPE%LL(TFP_FCLS%ICOD) .OR. &
 & QFPTYPE%LL(TFP_UGST%ICOD) .OR. QFPTYPE%LL(TFP_VGST%ICOD) .OR. &
 & QFPTYPE%LL(TFP_FGST%ICOD))  

!     ------------------------------------------------------------------

!*       2.    CALCULATIONS.
!              -------------

! Nothing is done if (LLSMC,LLASMC,LLICV,LLMTS,LLPBL,LLSTRMM,LLSHEAR,LLSRH,LLFLASH)=(F,F,F,F,F,F,F,F,F)
IF (LLSMC.OR.LLASMC.OR.LLICV.OR.LLMTS.OR.LLPBL.OR.LLSTRMM.OR.LLSHEAR.OR.LLSRH.OR.LLFLASH) THEN

  !*      2.1   MEMORY TRANSFERS

  ! GMV:
  IF (LLPBL .OR. LLMTS .OR. LLSTRMM .OR. LLSHEAR .OR. LLSRH .OR. LLFLASH) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KND
        ZUT0(JROF,JLEV)=PUT0(JROF,JLEV)*PGM(JROF)
        ZVT0(JROF,JLEV)=PVT0(JROF,JLEV)*PGM(JROF)
      ENDDO
    ENDDO
  ENDIF
  IF (LLPBL .OR. LLICV .OR. LLMTS .OR. LLFLASH) THEN
    DO JLEV=1,NFLEVG
      DO JROF=KST,KND
        ZTT0(JROF,JLEV)=PTT0(JROF,JLEV)
        IF (LNHDYN) ZPDT0(JROF,JLEV)=PSPDT0(JROF,JLEV)
      ENDDO
    ENDDO
  ENDIF

  ! GFL:
  IF (LLPBL .OR. LLICV .OR. LLMTS .OR. LLFLASH) THEN
    DO JLEV=1,NFLEVG
      IF (YDGFL%LLQ) THEN
        DO JROF=KST,KND
          ZQT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IQ)
        ENDDO
      ELSE
        ZQT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
      IF (YDGFL%LLL) THEN
        DO JROF=KST,KND
          ZLT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IL)
        ENDDO
      ELSE
        ZLT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
      !Fullpos inline case
      IF (NSTOP > 0_JPIM) THEN
        IF (YDGFL%LLLRAD) THEN
          DO JROF=KST,KND
            ZLRADT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%ILRAD)
          ENDDO
        ELSE
          ZLRADT0(KST:KND,JLEV)=0.0_JPRB
        ENDIF
      ELSE
      !Fullpos offline case
        IF (YDGFL%LLLRAD) THEN
          DO JROF=KST,KND
            ZLRADT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%ILRAD)
          ENDDO
        ELSEIF (YDGFL%LLL) THEN
          DO JROF=KST,KND
            ZLRADT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IL)
          ENDDO
        ELSE
          ZLRADT0(KST:KND,JLEV)=0.0_JPRB
        ENDIF
      ENDIF

      IF (YDGFL%LLI) THEN
        DO JROF=KST,KND
          ZIT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%II)
        ENDDO
      ELSE
        ZIT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
      !Fullpos inline case
      IF (NSTOP > 0_JPIM) THEN
        IF (YDGFL%LLIRAD) THEN
          DO JROF=KST,KND
            ZIRADT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IIRAD)
          ENDDO
        ELSE 
          ZIRADT0(KST:KND,JLEV)=0.0_JPRB
        ENDIF
      ELSE
      !Fullpos offline case
        IF (YDGFL%LLIRAD) THEN
          DO JROF=KST,KND
            ZIRADT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IIRAD)
          ENDDO
        ELSEIF (YDGFL%LLI) THEN
          DO JROF=KST,KND
            ZIRADT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%II)
          ENDDO
        ELSE
          ZIRADT0(KST:KND,JLEV)=0.0_JPRB
        ENDIF
      ENDIF   
      IF (YDGFL%LLRR) THEN
        DO JROF=KST,KND
          ZRRT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IRR)
        ENDDO
      ELSE
        ZRRT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
      IF (YDGFL%LLS) THEN
        DO JROF=KST,KND
          ZSNT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IS)
        ENDDO
      ELSE
        ZSNT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
      IF (YDGFL%LLG) THEN
        DO JROF=KST,KND
          ZGRHLT0(JROF,JLEV)=PGRHL(JROF,JLEV)
        ENDDO
      ELSE
        ZGRHLT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
      IF (YDGFL%LLA) THEN
        DO JROF=KST,KND
          ZAT0(JROF,JLEV)=PGFL(JROF,JLEV,YDIN%IA)
        ENDDO
      ELSE
        ZAT0(KST:KND,JLEV)=0.0_JPRB
      ENDIF
    ENDDO
  ENDIF

  !*      2.2   DYNAMICS

  ! GP.. and GNH.. routines must be called there;
  ! Calculations must remain consistent with CPG_GP and POS ones.
  
  IF (LLPBL.OR.LLICV.OR.LLMTS.OR.LLSTRMM.OR.LLSRH.OR.LLFLASH) THEN

    ! Compute R and Cp.
    CALL GPRCP(KPROMA,KST,KND,NFLEVG,PQ=ZQT0,PQI=ZIT0,&
     & PQL=ZLT0,PQR=ZRRT0,PQS=ZSNT0,PQG=ZGRHLT0,&
     & PCP=ZCP,PR=ZR,PKAP=ZUSL_KAP)

    ! Compute pressures.
    DO JROF=KST,KND 
      ZPRESH(JROF,NFLEVG)=EXP(PSPT0(JROF))
    ENDDO
    CALL GPPREH(KPROMA,KST,KND,NFLEVG,YRVAB%VAH,YRVAB%VBH,ZPRESH)
    CALL GPXYB(KPROMA,KST,KND,NFLEVG,YRVAB%VDELB,YRVAB%VC,ZPRESH,ZDELP,&
     & ZRDELP,ZLNPR,ZALPH,ZRTGR,ZRPRE,ZRPP)
    CALL GPPREF(KPROMA,KST,KND,NFLEVG,ZALPH,ZPRESH,ZPRESF)

    ! Compute NH pressure departure, and ZRRED=(pre/presim)*R
    IF (LLPBL.OR.LLSTRMM.OR.LLSRH) THEN
      IF (LNHDYN) THEN
        IF (LFPART2) THEN
          ! ZPDT0 contains pre-prehyd
          DO JLEV=1,NFLEVG
            DO JROF=KST,KND
              ZNHPREF(JROF,JLEV)=ZPDT0(JROF,JLEV)+ZPRESF(JROF,JLEV)
              ZRNHPPI(JROF,JLEV)=ZPRESF(JROF,JLEV)/ZNHPREF(JROF,JLEV)
              ZRRED(JROF,JLEV)=ZR(JROF,JLEV)*ZRNHPPI(JROF,JLEV)
            ENDDO
          ENDDO
        ELSE
          ! ZPDT0 contains pressure departure prognostic variable.
          CALL GNHPRE(KPROMA,NFLEVG,KST,KND,ZPDT0,ZPRESF, &
           & ZNHPREF,ZUSL_NHPPI,ZRNHPPI,ZUSL_PDEP)
          DO JLEV=1,NFLEVG
            DO JROF=KST,KND
              ZRRED(JROF,JLEV)=ZR(JROF,JLEV)*ZRNHPPI(JROF,JLEV)
            ENDDO
          ENDDO
        ENDIF
      ELSE
        ZNHPREF(KST:KND,1:NFLEVG)=ZPRESF(KST:KND,1:NFLEVG)
        ZRNHPPI(KST:KND,1:NFLEVG)=1.0_JPRB
        ZRRED(KST:KND,1:NFLEVG)=ZR(KST:KND,1:NFLEVG)
      ENDIF
    ENDIF

    ! Compute geopotential height.
    IF (LLPBL.OR.LLSTRMM.OR.LLSHEAR.OR.LLSRH) THEN
      IF (LNHDYN.AND.LRWSDLR) THEN
        ! "r/a", "a/r" and geopotential height.
        CALL GNHDLR(KPROMA,NFLEVG,KST,KND,ZTT0,ZR,ZRNHPPI, &
         & ZLNPR,ZALPH,POROG,ZRDLR,PHIH=ZGEOPH,PHIF=ZGEOPF)
      ELSE
        ! Geopotential height with surface orography=POROG
        ZGEOPH(KST:KND,NFLEVG)=POROG(KST:KND)
        CALL GPGEO(KPROMA,KST,KND,NFLEVG,ZGEOPH,ZGEOPF,ZTT0,ZRRED,ZLNPR,ZALPH)
      ENDIF
    ENDIF

    ! Convert model level pressures for LRWSDLR=T
    ! * post-processing done in part 2.3 only use true pressures "prehyd",
    !   and not the apparent variables used in the dynamics.
    ! * retrieval is done for prehyd, pre, and output data of GPXYB.
    IF (LNHDYN.AND.LRWSDLR) THEN
      ! Transform "presim" quantities into "prehyd" quantities.
      CALL GNHDLRB(KPROMA,KST,KND,.TRUE.,POROG,ZTT0,ZR,ZPDT0, &
       & ZPRESF,ZPRESH,ZDELP,ZRDELP,ZLNPR,ZALPH,PRPREHO=ZRPRE,PRPPO=ZRPP)
      ! From now on PSPT0 should not be used any more in PHYMFPOS.
      ! Use ZPRESH instead.
    ENDIF

  ENDIF

  DO JROF=KST,KND
    Z1SGM(JROF)=1.0_JPRB/PGM(JROF)
  ENDDO

  !*      2.3   POST-PROCESSING

  ! No GP.. and GNH.. routine call is allowed in this part

  ! * PBL fields:
  IF (LLPBL.OR.LLSHEAR) THEN
    CALL FPACHMT(KPROMA,KST,KND,NFLEVG,&
     & ZTT0,ZQT0,ZUT0,ZVT0,&
     & ZPRESH,ZPRESF,ZCP,ZR,ZGEOPH,ZGEOPF,&
     & PSD_VF(1,YSD_VF%YLSM%MP),PSP_RR(1,YSP_RR%YT%MP0),PSD_VF(1,YSD_VF%YZ0F%MP),&
     & PSD_VV(1,YSD_VV%YZ0H%MP),PSD_VF(1,YSD_VF%YVEG%MP),&
     & PSD_VV(1,YSD_VV%YARG%MP),PSD_VV(1,YSD_VV%YD2%MP),PSD_VV(1,YSD_VV%YIVEG%MP),PSD_VV(1,YSD_VV%YSAB%MP),&
     & PSP_SG(1,YSP_SG%YF%MP0),PSP_RR(1,YSP_RR%YW%MP0),PSP_RR(1,YSP_RR%YRR(4)%MP0),PSD_VV(1,YSD_VV%YHV%MP),&
     & ZQCLS,ZRHCLS,ZTCLS,ZUCLS, &
     & ZVCLS,ZFCLS,ZUGST,ZVGST,ZFGST,ZPCLS)

    IF (CFPFMT == 'MODEL') THEN
      CALL STORE_DATA(TFP_UCLS%ICOD,1,ZUCLS)
      CALL STORE_DATA(TFP_VCLS%ICOD,1,ZVCLS)
      CALL STORE_DATA(TFP_UGST%ICOD,1,ZUGST)
      CALL STORE_DATA(TFP_VGST%ICOD,1,ZVGST)
      CALL STORE_DATA(TFP_FCLS%ICOD,1,ZFCLS)
      CALL STORE_DATA(TFP_FGST%ICOD,1,ZFGST)
      CALL STORE_DATA(TFP_TCLS%ICOD,1,ZTCLS)
      CALL STORE_DATA(TFP_QCLS%ICOD,1,ZQCLS)
      CALL STORE_DATA(TFP_RCLS%ICOD,1,ZRHCLS)
    ENDIF
  ENDIF

  ! * Convection indices:
  IF (LLICV) THEN
    ICAPE=NFPCAPE
    IF (LFPCAPEX) THEN
      DO JROF=KST,KND
        ZPCLS(JROF)=EXP(PSPT0(JROF))
      ENDDO
      CALL FPCICA(KST,KND,KPROMA,NFLEVG,ICAPE,PXTCLS,ZPCLS,PXRHCLS,ZTT0,ZPRESF,&
       & ZQT0,ZUSL_KAP,ZCAPE,ZCIN,ILCL,IFCL,IEL)  
    ELSE
      CALL FPCICA(KST,KND,KPROMA,NFLEVG,ICAPE,ZTCLS,ZPCLS,ZRHCLS,ZTT0,ZPRESF,&
       & ZQT0,ZUSL_KAP,ZCAPE,ZCIN,ILCL,IFCL,IEL)  
    ENDIF
    CALL STORE_DATA(TFP_CAPE%ICOD,1,ZCAPE)
    CALL STORE_DATA(TFP_CIEN%ICOD,1,ZCIN)
    ZLCL=ILCL
    ZFCL=IFCL
    ZEL=IEL
    CALL STORE_DATA(TFP_LCL%ICOD ,1,ZLCL)
    CALL STORE_DATA(TFP_FCL%ICOD ,1,ZFCL)
    CALL STORE_DATA(TFP_EL%ICOD ,1,ZEL)
  ENDIF

  ! * lightening diagnostics                                                                                                                
  IF (LLFLASH) THEN                                          
    CALL FPDIAGFLASH(KST,KND,KPROMA,NFLEVG,&                                                                                              
       & ZLT0,ZIT0,ZRRT0,ZSNT0,ZGRHLT0,ZDELP,ZTT0,PWW,PSD_VF(1,YSD_VF%YLSM%MP),ZFLASH)
    CALL STORE_DATA(TFP_FLASH%ICOD,1,ZFLASH)                                                                                            
  ENDIF 
  
  ! * compute storm motion diagnostics
  IF (LLSTRMM) THEN 
    CALL FPSTRMM(KST,KND,KPROMA,NFLEVG,& 
          & ZUT0,ZVT0,ZDELP,ZPRESF,POROG,ZGEOPF,ZSTRMMU,ZSTRMMV) 
    CALL STORE_DATA(TFP_STRMMU%ICOD,1,ZSTRMMU)
    CALL STORE_DATA(TFP_STRMMV%ICOD,1,ZSTRMMV)
  ENDIF
 
  ! * compute deep layer and low level shear  
  IF (LLSHEAR) THEN
    CALL FPSHEAR(KST,KND,KPROMA,NFLEVG,&
         & ZUT0,ZVT0,ZUCLS,ZVCLS,POROG,ZGEOPF,ZDLS,ZLLS) 
    CALL STORE_DATA(TFP_DLS%ICOD,1,ZDLS)
    CALL STORE_DATA(TFP_LLS%ICOD,1,ZLLS)
  ENDIF
 
  ! * compute storm relative helicity (storm motion needed) 
  IF (LLSRH.AND.LLSTRMM) THEN
    CALL FPSRH(KST,KND,KPROMA,NFLEVG, &
         & ZUT0,ZVT0,ZDELP,ZPRESF,POROG,ZGEOPF,ZSTRMMU,ZSTRMMV,ZSRH)
    CALL STORE_DATA(TFP_SRH%ICOD,1,ZSRH)
  ENDIF

  ! * Surface moisture convergence:
  IF (LLSMC) THEN
    DO JROF=KST,KND
      ZUSMC(JROF)=-ZQCLS(JROF)*ZUCLS(JROF)
      ZVSMC(JROF)=-ZQCLS(JROF)*ZVCLS(JROF)
    ENDDO
    CALL STORE_DATA(TFP_SMC%ICOD,1,ZUSMC)
    CALL STORE_DATA(TFP_SMC%ICOD,2,ZVSMC)
  ENDIF

  IF (LLASMC) THEN
    IF (LMOCONVAR) THEN
      ! specific mocon computation for variational diagnostic purposes
      DO JROF=KST,KND
        ZUASM(JROF)=-ZQT0(JROF,NFLEVG-1)*PUT0(JROF,NFLEVG)*PGM(JROF)
        ZVASM(JROF)=-ZQT0(JROF,NFLEVG-1)*PVT0(JROF,NFLEVG)*PGM(JROF)
      ENDDO
    ELSE
      DO JROF=KST,KND
        ! Compute specific moisture at PBL height 
        IF(LNEIGE) THEN
          ZDELTA=MAX(0.0_JPRB,SIGN(1.0_JPRB,RTT-PXTCLS(JROF)))
        ELSE
          ZDELTA=0.0_JPRB
        ENDIF
        ZES=FOEW(MAX(PXTCLS(JROF),1.0_JPRB),ZDELTA)
        ZXQCLS(JROF)=-PXRHCLS(JROF)*ZES &
         & /(RETV*PXRHCLS(JROF)*ZES-ZPCLS(JROF)*(RETV+1.0_JPRB))  
        ! Compute moisture convergence not reduced :
        ZUASM(JROF)=-ZXQCLS(JROF)*PXUCLS(JROF)*PGM(JROF)
        ZVASM(JROF)=-ZXQCLS(JROF)*PXVCLS(JROF)*PGM(JROF)
      ENDDO
    ENDIF
    CALL STORE_DATA(TFP_ASMC%ICOD,1,ZUASM)
    CALL STORE_DATA(TFP_ASMC%ICOD,2,ZVASM)
  ENDIF

  ! * Model Radiances:
  IF (LLMTS) THEN
    ! this O3 initialization is scratched by the  RTTOV climatology in MTS_PHYS
    ZQO3(:,:) = 1.E-9_JPRB 
    ZTS0(:)=MIN(MAX(PSP_RR(:,YSP_RR%YT%MP0),200._JPRB),350._JPRB)
    IF (.NOT. LLPBL) THEN
      ! these local are necessary for the points belonging to the unphysical extension 
      !  domain of ALADIN or AROME 
      ZUCLS=MIN(MAX(PXUCLS,-70._JPRB),70._JPRB)
      ZVCLS=MIN(MAX(PXVCLS,-70._JPRB),70._JPRB)
      ZTCLS=MIN(MAX(PXTCLS,200._JPRB),350._JPRB)
      ZQCLS=MIN(MAX(PXQCLS,1.E-9_JPRB),0.04_JPRB)
    ENDIF 

    CALL MTS_PHYS(KST,KND,KPROMA,NFLEVG,PGEMU,PGELAM,POROG,ZPRESH,ZPRESF,ZLNPR,ZALPH,&
     &ZUT0,ZVT0,ZTT0,ZQT0,ZQO3,ZLRADT0,ZIRADT0,ZAT0,ZTS0,PSD_VF(1,YSD_VF%YLSM%MP),ZUCLS,ZVCLS,&
     &ZTCLS,ZQCLS,&
     &ZTBMSAT7,ZTBMSAT8,ZTBMSAT9,ZTBGOES11,ZTBGOES12,ZTBMTSAT1)
    
    IF(QFPTYPE%LL(TFP_MSAT7C1%ICOD)) CALL STORE_DATA(TFP_MSAT7C1%ICOD,1,ZTBMSAT7(:,1))
    IF(QFPTYPE%LL(TFP_MSAT7C2%ICOD)) CALL STORE_DATA(TFP_MSAT7C2%ICOD,1,ZTBMSAT7(:,2))
    IF(QFPTYPE%LL(TFP_MSAT8C1%ICOD)) CALL STORE_DATA(TFP_MSAT8C1%ICOD,1,ZTBMSAT8(:,1))
    IF(QFPTYPE%LL(TFP_MSAT8C2%ICOD)) CALL STORE_DATA(TFP_MSAT8C2%ICOD,1,ZTBMSAT8(:,2))
    IF(QFPTYPE%LL(TFP_MSAT8C3%ICOD)) CALL STORE_DATA(TFP_MSAT8C3%ICOD,1,ZTBMSAT8(:,3))
    IF(QFPTYPE%LL(TFP_MSAT8C4%ICOD)) CALL STORE_DATA(TFP_MSAT8C4%ICOD,1,ZTBMSAT8(:,4))
    IF(QFPTYPE%LL(TFP_MSAT8C5%ICOD)) CALL STORE_DATA(TFP_MSAT8C5%ICOD,1,ZTBMSAT8(:,5))
    IF(QFPTYPE%LL(TFP_MSAT8C6%ICOD)) CALL STORE_DATA(TFP_MSAT8C6%ICOD,1,ZTBMSAT8(:,6))
    IF(QFPTYPE%LL(TFP_MSAT8C7%ICOD)) CALL STORE_DATA(TFP_MSAT8C7%ICOD,1,ZTBMSAT8(:,7))
    IF(QFPTYPE%LL(TFP_MSAT8C8%ICOD)) CALL STORE_DATA(TFP_MSAT8C8%ICOD,1,ZTBMSAT8(:,8))
    IF(QFPTYPE%LL(TFP_MSAT9C1%ICOD)) CALL STORE_DATA(TFP_MSAT9C1%ICOD,1,ZTBMSAT9(:,1))
    IF(QFPTYPE%LL(TFP_MSAT9C2%ICOD)) CALL STORE_DATA(TFP_MSAT9C2%ICOD,1,ZTBMSAT9(:,2))
    IF(QFPTYPE%LL(TFP_MSAT9C3%ICOD)) CALL STORE_DATA(TFP_MSAT9C3%ICOD,1,ZTBMSAT9(:,3))
    IF(QFPTYPE%LL(TFP_MSAT9C4%ICOD)) CALL STORE_DATA(TFP_MSAT9C4%ICOD,1,ZTBMSAT9(:,4))
    IF(QFPTYPE%LL(TFP_MSAT9C5%ICOD)) CALL STORE_DATA(TFP_MSAT9C5%ICOD,1,ZTBMSAT9(:,5))
    IF(QFPTYPE%LL(TFP_MSAT9C6%ICOD)) CALL STORE_DATA(TFP_MSAT9C6%ICOD,1,ZTBMSAT9(:,6))
    IF(QFPTYPE%LL(TFP_MSAT9C7%ICOD)) CALL STORE_DATA(TFP_MSAT9C7%ICOD,1,ZTBMSAT9(:,7))
    IF(QFPTYPE%LL(TFP_MSAT9C8%ICOD)) CALL STORE_DATA(TFP_MSAT9C8%ICOD,1,ZTBMSAT9(:,8))
    IF(QFPTYPE%LL(TFP_GOES11C1%ICOD)) CALL STORE_DATA(TFP_GOES11C1%ICOD,1,ZTBGOES11(:,1))
    IF(QFPTYPE%LL(TFP_GOES11C2%ICOD)) CALL STORE_DATA(TFP_GOES11C2%ICOD,1,ZTBGOES11(:,2))
    IF(QFPTYPE%LL(TFP_GOES11C3%ICOD)) CALL STORE_DATA(TFP_GOES11C3%ICOD,1,ZTBGOES11(:,3))
    IF(QFPTYPE%LL(TFP_GOES11C4%ICOD)) CALL STORE_DATA(TFP_GOES11C4%ICOD,1,ZTBGOES11(:,4))
    IF(QFPTYPE%LL(TFP_GOES12C1%ICOD)) CALL STORE_DATA(TFP_GOES12C1%ICOD,1,ZTBGOES12(:,1))
    IF(QFPTYPE%LL(TFP_GOES12C2%ICOD)) CALL STORE_DATA(TFP_GOES12C2%ICOD,1,ZTBGOES12(:,2))
    IF(QFPTYPE%LL(TFP_GOES12C3%ICOD)) CALL STORE_DATA(TFP_GOES12C3%ICOD,1,ZTBGOES12(:,3))
    IF(QFPTYPE%LL(TFP_GOES12C4%ICOD)) CALL STORE_DATA(TFP_GOES12C4%ICOD,1,ZTBGOES12(:,4))
    IF(QFPTYPE%LL(TFP_MTSAT1C1%ICOD)) CALL STORE_DATA(TFP_MTSAT1C1%ICOD,1,ZTBMTSAT1(:,1))
    IF(QFPTYPE%LL(TFP_MTSAT1C2%ICOD)) CALL STORE_DATA(TFP_MTSAT1C2%ICOD,1,ZTBMTSAT1(:,2))
    IF(QFPTYPE%LL(TFP_MTSAT1C3%ICOD)) CALL STORE_DATA(TFP_MTSAT1C3%ICOD,1,ZTBMTSAT1(:,3))
    IF(QFPTYPE%LL(TFP_MTSAT1C4%ICOD)) CALL STORE_DATA(TFP_MTSAT1C4%ICOD,1,ZTBMTSAT1(:,4))
  ENDIF

ENDIF

IF (LHOOK) CALL DR_HOOK('PHYMFPOS',1,ZHOOK_HANDLE)

!------------------------------------------------------------------
CONTAINS

SUBROUTINE STORE_DATA(K,KPOS,PDATA)

INTEGER(KIND=JPIM), INTENT(IN) :: K             ! Field code/index
INTEGER(KIND=JPIM), INTENT(IN) :: KPOS          ! Relative adress of input data
REAL(KIND=JPRB),    INTENT(IN) :: PDATA(KPROMA) ! input data

INTEGER(KIND=JPIM) :: ILOC ! Absolute adress of data in output array
REAL(KIND=JPRB) :: ZHOOK_HANDLE

IF (LHOOK) CALL DR_HOOK('PHYMFPOS:STORE_DATA',0,ZHOOK_HANDLE)
IF (QFPTYPE%LL(K)) THEN
  SELECT CASE (QFPTYPE%ISF(K))
  CASE (0)
    ! Not fitted data 
    ILOC=QFPTYPE%IGPX(K)
    SELECT CASE (ABS(TFP_DYNDS(K)%IORDR))
    CASE (0)
      DO JROF=KST,KND
        PAUX(JROF,ILOC) = PDATA(JROF)
      ENDDO
    CASE (1)
      DO JROF=KST,KND
        PAUX(JROF,ILOC) = PDATA(JROF)*Z1SGM(JROF)
      ENDDO
    CASE (2)
      DO JROF=KST,KND
        PAUX(JROF,ILOC) = PDATA(JROF)*(Z1SGM(JROF)**2)
      ENDDO
    END SELECT
  CASE (1:)
    ! Fitted data 
    IF (KPOS > 0 .AND. KPOS <= QFPTYPE%ISKP(K)) THEN
      ILOC=QFPTYPE%IGT1(K,KPOS)
    ELSE
      CALL ABOR1('PHYMFPOS.STORE_DATA : INTERNAL ERROR KPOS')
    ENDIF
    SELECT CASE (ABS(TFP_DYNDS(K)%IORDR))
    CASE (0)
      DO JROF=KST,KND
        PGPP(JROF,ILOC) = PDATA(JROF)
      ENDDO
    CASE (1)
      DO JROF=KST,KND
        PGPP(JROF,ILOC) = PDATA(JROF)*Z1SGM(JROF)
      ENDDO
    CASE (2)
      DO JROF=KST,KND
        PGPP(JROF,ILOC) = PDATA(JROF)*(Z1SGM(JROF)**2)
      ENDDO
    END SELECT
  END SELECT 
ENDIF 
IF (LHOOK) CALL DR_HOOK('PHYMFPOS:STORE_DATA',1,ZHOOK_HANDLE)

END SUBROUTINE STORE_DATA

!     ------------------------------------------------------------------

END SUBROUTINE PHYMFPOS

