#ifdef RS6K
@PROCESS NOCHECK
#endif
SUBROUTINE VPOS(KST,KEND,KSTGLO,KGFL,KGMV,KGMVS,CDCONF,YDIN_GFL,YDGFL,YDIN_GMV, &
 & YDGSGEOM,YDOROG,PGMV,PGMVS,PGFL)

!**** *VPOS*  - VERTICAL POST-PROCESSING - FULL POS

! PURPOSE.
! --------
!  Full-POS interface to POS.

! INTERFACE.
! ----------
!   *CALL* *VPOS*

! EXPLICIT ARGUMENTS
! --------------------
! KST,KEND     : First and last adress of computation in NPROMA-sized arrays
! KSTGLO       : Start adress of computation in NGPTOT-sized arrays
! KGFL         : Number of GFL fields in PGFL
! KGMV         : Number of GMV fields in PGMV
! KGMVS        : Number of GMVS fields in PGMVS
! CDCONF       : Configuration of work (used in POS to know the vertical coord of interpolation)
! YDIN_GFL     : Pointers allowing to retrieve individual fields in PGFL
! YDGFL        : says if GFL is active or not
! YDIN_GMV     : Pointers allowing to retrieve individual fields in PGMV and PGMVS
! YDGSGEOM     : Grid point geometry
! YDOROG       : Orography arrays
! PGMV         : GMV fields
! PGMVS        : GMVS fields
! PGFL         : GFL fields

! IMPLICIT ARGUMENTS
! --------------------

! METHOD.
! -------
!   SEE DOCUMENTATION

! EXTERNALS.
! ----------

!     REFERENCE.
!     ----------
!        ECMWF Research Department documentation of the IFS

!     AUTHOR.
!     -------
!      RYAD EL KHATIB *METEO-FRANCE*
!      ORIGINAL : 94-04-08

!     MODIFICATIONS.
!     --------------
!      R. El Khatib : (Fev 2002) Fullpos B-level distribution + remove IO scheme
!      R. El Khatib : 03-08-12 gfl
!      R. El Khatib : 03-04-17 Fullpos improvments
!      M.Hamrud      01-Oct-2003 CY28 Cleaning
!      M.Hamrud      10-Jan-2004 CY28R1 Cleaning
!      R. Engelen      15-Mar-2005 Greenhouse Gases
!      A. Untch      15-Mar-2005 Aerosols
!      J. Flemming   11-Apr-2005 Aerosol replaced with GRG
!      G. Hello     : 05-01-26 arome gfl
!      R. El Ouaraini & R. El Khatib : 05-07-28 Full-pos of CUF fields
!      M. Bellus    : 29-Sep-2006 Prognostic convection and MAC scheme (ALARO-0)
!      S. Serrar    : 07-09-2006 include tracers used for diagnostics only (GEMS)
!      M.Hamrud      01-Jul-2006 Revised surface fields
!      S. Serrar    : 22-03-2007 Post-Process ERA40 diagnostic fields
!      S. Serrar    : 17-07-2007 Two GFL fields methane related
!      N. Wedi and K. Yessad (Jan 2008): different dev for NH model and PC scheme
!      K. Yessad (Aug 2009): rewrite vertical interpolator in post-processing.
!      G.Mozdzynski (Feb 2011): OOPS cleaning, use of derived types TGSGEOM and TOROG
!      H. Hersbach  : 01-04-2011 auxiliary diagnostic radiation fields
!      K. Yessad (Dec 2011): various contributions.
!      R. El Khatib : 23-Mar-2012 Fix bounds checking issues
!     ------------------------------------------------------------------

USE PARKIND1  ,ONLY : JPIM     ,JPRB
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE YOMCT0   , ONLY : LFPART2  ,LNHDYN
USE YOMDYNA  , ONLY : LRWSDLR
USE YOMDYN   , ONLY : RCORPDH  ,RCORPDF
USE YOMDIM   , ONLY : NFLEVG   ,NPROMA   ,NPROMVC  ,NPROMDLR, NPPM
USE YOMFPDIM , ONLY : NFPGT1   ,NFPAUXB
USE YOMAUXB  , ONLY : GAUXBUF
USE YOMPHY   , ONLY : LMPHYS
USE YOMCST   , ONLY : RV       ,RCPV     ,RETV     ,&
 & RCW      ,RCS      ,RLVTT    ,RLSTT    ,RTT      ,&
 & RALPW    ,RBETW    ,RGAMW    ,RALPS    ,RBETS    ,&
 & RGAMS    ,RALPD    ,RBETD    ,RGAMD    ,RG
USE YOMSTA   , ONLY : RDTDZ1   ,NLEXTRAP
USE YOMGEM   , ONLY : YRVAB
USE YOMXFUB  , ONLY : XFUBUF
USE YOMXFU   , ONLY : NFDXFU
USE PTRXFU   , ONLY : MXRHCLS   ,MXTCLS  ,MXUCLS   ,MXVCLS, MXQCLS
USE YOMFPG   , ONLY : FPVALH   ,FPVBH    ,NFPLEV
USE YOMVPOS  , ONLY : XFPLEV   ,NXFPLEV  ,QFPTYPE
USE YOMOMPDIST , ONLY : GPP
USE SURFACE_FIELDS_MIX , ONLY : SP_RR, YSP_RR,&
 & SD_VF, YSD_VF, SD_VV, SD_XA, SD_XR, SP_SG
USE YOMMCUF  , ONLY : RMCUFGP  ,NCUFNR
USE TYPE_GFLFLDS, ONLY: TYPE_IGFLFLDD,TYPE_LGFLFLDD
USE TYPE_GMVS, ONLY : TYPE_T0
USE YOMAFN   , ONLY : TFP_T    ,TFP_Q
USE YOMFPC   , ONLY : LFPCLSTOGMV
USE YOMGC    , ONLY : TGSGEOM  ,TOROG
USE INTDYN_MOD,ONLY : YYTXYB
!     ------------------------------------------------------------------

IMPLICIT NONE

INTEGER(KIND=JPIM),INTENT(IN)    :: KST 
INTEGER(KIND=JPIM),INTENT(IN)    :: KEND 
INTEGER(KIND=JPIM),INTENT(IN)    :: KSTGLO 
INTEGER(KIND=JPIM),INTENT(IN)    :: KGFL
INTEGER(KIND=JPIM),INTENT(IN)    :: KGMV
INTEGER(KIND=JPIM),INTENT(IN)    :: KGMVS
CHARACTER(LEN=1)  ,INTENT(IN)    :: CDCONF 
TYPE(TYPE_IGFLFLDD),INTENT(IN)   :: YDIN_GFL
TYPE(TYPE_LGFLFLDD),INTENT(IN)   :: YDGFL
TYPE(TYPE_T0)     ,INTENT(IN)    :: YDIN_GMV
TYPE(TGSGEOM)     ,INTENT(IN)    :: YDGSGEOM
TYPE(TOROG)       ,INTENT(IN)    :: YDOROG
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMV(NPROMA,NFLEVG,KGMV) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGMVS(NPROMA,KGMVS) 
REAL(KIND=JPRB)   ,INTENT(IN)    :: PGFL(NPROMA,NFLEVG,KGFL) 

!     ------------------------------------------------------------------

INTEGER(KIND=JPIM) :: JROF, JLEV, IBL
INTEGER(KIND=JPIM) :: ICODLOC,ILOC

REAL(KIND=JPRB) :: ZDTDZSG    ! dT/g.dZ

REAL(KIND=JPRB) :: ZTSO(NPROMA)           ! Output surface temperature
REAL(KIND=JPRB) :: ZIOROG(NPROMA)         ! Interpolated surface geopotential
REAL(KIND=JPRB) :: ZTSI(NPROMA)           ! Interpolated surface temperature
REAL(KIND=JPRB) :: ZXFU(NPROMA,0:NFDXFU)  ! Instantaneous fields
REAL(KIND=JPRB) :: ZAUX(NPROMA,NFPAUXB)   !
REAL(KIND=JPRB) :: ZOROG(NPROMA),ZOROGL(NPROMA),ZOROGM(NPROMA)
REAL(KIND=JPRB) :: ZGEMU(NPROMA),ZGELAM(NPROMA)
REAL(KIND=JPRB) :: ZRCORI(NPROMA),ZGM(NPROMA),ZGRHL(NPROMA,NFLEVG),ZWW(NPROMA,NFLEVG)
REAL(KIND=JPRB) :: ZMCUF(NPROMA,NCUFNR)
REAL(KIND=JPRB) :: ZEW

REAL(KIND=JPRB),ALLOCATABLE :: ZIPH(:,:), ZIPF(:,:), ZTSEA(:)
REAL(KIND=JPRB) :: ZHOOK_HANDLE

!     ------------------------------------------------------------------

#include "ctstar.intfb.h"
#include "gphpre.intfb.h"
#include "phymfpos.intfb.h"
#include "pos.intfb.h"
#include "sc2rdg.intfb.h"
#include "sc2wrg.intfb.h"
#include "fcttrm.h"

!     ------------------------------------------------------------------

IF (LHOOK) CALL DR_HOOK('VPOS',0,ZHOOK_HANDLE)

!     ------------------------------------------------------------------

!*       1.    PREPARATIONS
!              ------------

IBL=(KSTGLO-1)/NPROMA+1

ZDTDZSG=RDTDZ1/RG

DO JROF=KST,KEND
  ZOROG (JROF)=YDOROG%OROG (JROF)
  ZRCORI(JROF)=YDGSGEOM%RCORI(JROF)
  ZGM   (JROF)=YDGSGEOM%GM   (JROF)
  ZGEMU (JROF)=YDGSGEOM%GEMU (JROF)
  ZGELAM (JROF)=YDGSGEOM%GELAM (JROF)
  ZOROGL(JROF)=YDOROG%OROGL(JROF)
  ZOROGM(JROF)=YDOROG%OROGM(JROF)
  IF (LFPART2) THEN
    ZIOROG(JROF)=SD_VF(JROF,YSD_VF%YFP1%MP,IBL)
  ELSE
    ZIOROG(JROF)=ZOROG(JROF)
  ENDIF
ENDDO

IF (LMPHYS) THEN
  DO JROF=KST,KEND
    IF (LFPART2) THEN
      ZTSI(JROF)=SP_RR(JROF,YSP_RR%YFP1%MP0,IBL)
    ELSE
      ZTSI(JROF)=SP_RR(JROF,YSP_RR%YT%MP0,IBL)
    ENDIF
    ZTSO(JROF)=SP_RR(JROF,YSP_RR%YT%MP0,IBL)
  ENDDO
ELSE
  ! ECMWF: only one T profile
  !  ZTSI extrapolated from T(NLEXTRAP)
  !  ZTSO not used
  ! Compute surface temperatures :
  ALLOCATE(ZIPH(NPROMA,0:NFLEVG))
  ALLOCATE(ZIPF(NPROMA,NFLEVG))
  ALLOCATE(ZTSEA(NPROMA))
  DO JROF=KST,KEND
    ZIPH(JROF,NFLEVG)=EXP(PGMVS(JROF,YDIN_GMV%MSP))
  ENDDO
  CALL GPHPRE(NPROMA,NFLEVG,KST,KEND,YRVAB,ZIPH,PRESF=ZIPF)
  IF (LNHDYN.AND.LRWSDLR) THEN
    ! ZIPH and ZIPF contain "presim", but CTSTAR requires something
    ! looking like "prehyd": simplified conversion done using a reference
    ! profile of "presim/prehyd".
    DO JROF=KST,KEND
      ZIPH(JROF,NFLEVG)=(1.0_JPRB/EXP(RCORPDH(NFLEVG)))*ZIPH(JROF,NFLEVG)
      ZIPF(JROF,NLEXTRAP)=(1.0_JPRB/EXP(RCORPDF(NLEXTRAP)))*ZIPF(JROF,NLEXTRAP)
    ENDDO
  ENDIF
  CALL CTSTAR(NPROMA,KST,KEND,PGMV(1,NLEXTRAP,YDIN_GMV%MT),ZIPH(1,NFLEVG), &
   & ZIPF(1,NLEXTRAP),ZIOROG,ZTSI,ZTSEA)  
  DO JROF=KST,KEND
    ZTSO(JROF)=ZTSEA(JROF)+ZDTDZSG*ZOROG(JROF)
  ENDDO
  DEALLOCATE(ZIPH)
  DEALLOCATE(ZIPF)
  DEALLOCATE(ZTSEA)
ENDIF

IF (NFDXFU > 0) THEN
  CALL SC2RDG(XFUBUF,KSTGLO,KST,KEND,ZXFU(1:NPROMA,1:NFDXFU))
ENDIF

IF (NCUFNR > 0) THEN
  DO JLEV=1,NCUFNR
    DO JROF=KST,KEND
      ZMCUF(JROF,JLEV)=RMCUFGP(KSTGLO+JROF-1,JLEV)
    ENDDO
  ENDDO
ENDIF

!*       2.    PERFORM VERTICAL POST-PROCESSING
!              --------------------------------

! ky: passing PGMV and PGMVS in one step to POS expected for CY40.
CALL POS(NPROMA,NPROMVC,NPROMDLR,KST,KEND,&
 & NXFPLEV,KGFL,NFPGT1,NFPAUXB,NFPLEV,NFLEVG,NPPM,CDCONF,&
 & YDIN_GFL,YDGFL,FPVALH,FPVBH,ZRCORI,ZGM,ZOROG,ZOROGL,ZOROGM,ZIOROG,&
 & XFPLEV,ZTSI,ZTSO,PGMV(1,1,YDIN_GMV%MEDOT), &
 & PGMV(1,1,YDIN_GMV%MU),PGMV(1,1,YDIN_GMV%MUL),PGMV(1,1,YDIN_GMV%MDIV),&
 & PGMV(1,1,YDIN_GMV%MV),PGMV(1,1,YDIN_GMV%MVL),PGMV(1,1,YDIN_GMV%MVOR),&
 & PGMV(1,1,YDIN_GMV%MT),PGMV(1,1,YDIN_GMV%MTL),PGMV(1,1,YDIN_GMV%MTM),&
 & PGMV(1,1,YDIN_GMV%MSPD),PGMV(1,1,YDIN_GMV%MSPDL),PGMV(1,1,YDIN_GMV%MSPDM),&
 & PGMV(1,1,YDIN_GMV%MSVD),PGMV(1,1,YDIN_GMV%MNHX),&
 & PGMVS(1,YDIN_GMV%MSP),PGMVS(1,YDIN_GMV%MSPL),PGMVS(1,YDIN_GMV%MSPM),PGFL,&
 & SD_XA(:,:,:,IBL),SD_XR(:,:,:,IBL),ZMCUF(:,:),&
 & ZWW,&
 & GPP(:,:,IBL),ZAUX)

!*       3.    PERFORM PHYSICO-DYNAMIC POST-PROCESSING OVER THE MODEL SURFACE
!              --------------------------------------------------------------

ZGRHL(:,:)=0._JPRB
IF(YDGFL%LLG) THEN
  DO JLEV=1,NFLEVG
    DO JROF=KST,KEND
      IF (YDGFL%LLH) THEN
        ZGRHL(JROF,JLEV)=PGFL(JROF,JLEV,YDIN_GFL%IG)+PGFL(JROF,JLEV,YDIN_GFL%IH)
      ELSE
        ZGRHL(JROF,JLEV)=PGFL(JROF,JLEV,YDIN_GFL%IG)
      ENDIF
    ENDDO
  ENDDO
ENDIF

CALL PHYMFPOS(NPROMA,NPROMDLR,KST,KEND,NFPAUXB,NFPGT1,KGFL,YDIN_GFL,YDGFL,&
 & ZOROG,ZGM,ZGEMU,ZGELAM,PGMV(1,1,YDIN_GMV%MU),PGMV(1,1,YDIN_GMV%MV),&
 & PGMV(1,1,YDIN_GMV%MT),PGMV(1,1,YDIN_GMV%MSPD),ZGRHL,ZWW,PGFL,PGMVS(1,YDIN_GMV%MSP),&
 & SP_RR(:,:,IBL),SP_SG(:,:,IBL),SD_VF(:,:,IBL),SD_VV(:,:,IBL),& 
 & ZXFU(1,MXTCLS),ZXFU(1,MXRHCLS),ZXFU(1,MXQCLS),&
 & ZXFU(1,MXUCLS),ZXFU(1,MXVCLS),ZAUX,GPP(:,:,IBL))  

! Copy CLS fields T and Q into last model level, used for varpack when
! the last model level correspond approximatively to 2m height.
IF (LFPCLSTOGMV) THEN
  IF (LFPART2) THEN

    ICODLOC=TFP_T%ICOD
    ILOC=QFPTYPE%IGT1(ICODLOC,1)+NXFPLEV-1
    GPP(KST:KEND,ILOC,IBL)=ZXFU(KST:KEND,MXTCLS)
      
    ICODLOC=TFP_Q%ICOD
    ILOC=QFPTYPE%IGT1(ICODLOC,1)+NXFPLEV-1
    DO JROF=KST,KEND
      ZEW=FOEW(ZXFU(JROF,MXTCLS),0._JPRB)
      GPP(JROF,ILOC,IBL)=(ZXFU(JROF,MXRHCLS)*ZEW)/(EXP(PGMVS(JROF,YDIN_GMV%MSP))*(RETV+1.0_JPRB)-RETV*ZEW*ZXFU(JROF,MXRHCLS))
    ENDDO

  ENDIF
ENDIF

!*       4.    WRITE OUT GRIDPOINT FIELDS INTO BUFFERS/WORKFILES
!              -------------------------------------------------

IF (NFPAUXB > 0) THEN 
  CALL SC2WRG(GAUXBUF,KSTGLO,KST,KEND,ZAUX)
ENDIF

!     ------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('VPOS',1,ZHOOK_HANDLE)

END SUBROUTINE VPOS

